/*
 * This file is part of RDC-ANALYTIC.
 *
 * RDC-ANALYTIC Protein Backbone Structure Determination Software Version 1.0
 * Copyright (C) 2001-2012 Bruce Donald Lab, Duke University
 *
 * RDC-ANALYTIC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * RDC-ANALYTIC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, see:
 *     <http://www.gnu.org/licenses/>.
 *
 * There are additional restrictions imposed on the use and distribution of this
 * open-source code, including: (A) this header must be included in any
 * modification or extension of the code; (B) you are required to cite our
 * papers in any publications that use this code. The citation for the various
 * different modules of our software, together with a complete list of
 * requirements and restrictions are found in the document license.pdf enclosed
 * with this distribution.
 *
 * Contact Info:
 *     Bruce R. Donald
 *     Duke University
 *     Department of Computer Science
 *     Levine Science Research Center (LSRC)
 *     Durham, NC 27708-0129
 *     USA
 *     email: www.cs.duke.edu/brd/
 *
 * <signature of Bruce Donald>, August 04, 2012
 * Bruce R. Donald, Professor of Computer Science and Biochemistry
 */

/**
 * @version       1.0.1, August 04, 2012
 * @author        Chittaranjan Tripathy (2007-2012)
 * @email         chittu@cs.duke.edu
 * @organization  Duke University
 */

/**
 * Package specification
 */
package analytic;

/**
 * Import statement(s)
 */
import java.io.*;

/**
 * This class is provided to produce verbose output while debugging other classes.
 */
public class myRedPencil {
    
    private boolean __on;    
    
    private final String __global_red_pencil = myMiscConstants.globalRedPencil;
       
    public myRedPencil(boolean on) {
        if (__global_red_pencil.equalsIgnoreCase("ON")) {
            __on = true;
            return;
        } else if (__global_red_pencil.equalsIgnoreCase("OFF")) {
            __on = false;
            return;
        }
        __on = on;
    }
    
    public myRedPencil(int on) {
        if (__global_red_pencil.equalsIgnoreCase("ON")) {
            __on = true;
            return;
        } else if (__global_red_pencil.equalsIgnoreCase("OFF")) {
            __on = false;
            return;
        }       
        if (on == 1) {
            __on = true;
        } else if (on == 0) {
            __on = false;
        } else {
            System.out.println("Error: the argument to the method myRedPencil() must be 0 or 1");
            System.exit(1);
        }
    }
    
    public myRedPencil(String s) {
        if (__global_red_pencil.equalsIgnoreCase("ON")) {
            __on = true;
            return;
        } else if (__global_red_pencil.equalsIgnoreCase("OFF")) {
            __on = false;
            return;
        }
        s = s.trim().toUpperCase();
        switch (s) {
            case "TRUE":
            case "ON":
                __on = true; 
                break;
            case "FALSE":
            case "OFF":
                __on = false;
                break;
            default: {
                System.out.println("Error: the argument to the method myRedPencil() must be true/false/on/off");
                System.exit(1);
            }
        }
    }
    
    public boolean isOn() {
        return __on;
    }
    
    public void setOn() {
       if (__global_red_pencil.equalsIgnoreCase("ON") || __global_red_pencil.equalsIgnoreCase("OFF")) {
           System.out.println("Error: Cannot set the red pencil since the global red pencil status is ON/OFF. It has to have Don'tCare status");
           System.exit(1);
       }
       __on = true;
    }
    
    public void setOff() {
       if (__global_red_pencil.equalsIgnoreCase("ON") || __global_red_pencil.equalsIgnoreCase("OFF")) {
           System.out.println("Error: Cannot set the red pencil since the global red pencil status is ON/OFF. It has to have Don'tCare status");
           System.exit(1);
       }        
        __on = false;
    }
    
    public static void main(String... args) {
        myRedPencil mrp = new myRedPencil(false);
        System.out.println(mrp.isOn());
        
    }
}
